@@ -57,7 +57,7 @@
 #ifndef CDDA_DEVICE
 # ifdef HAVE_SYS_CDIO_H
 #  ifdef __FreeBSD__
-#   define CDDA_DEVICE "/dev/acd0c"
+#   define CDDA_DEVICE "/dev/acd0"
 #  elif defined __OpenBSD__
 #   define CDDA_DEVICE "/dev/cd0c"
 #  else
@@ -278,7 +278,6 @@ static void cleanup(void)
 		struct timeout *t = timeout_list->data;
 		gtk_timeout_remove(t->id);
 		stop_timeout(t);
-		timeout_destroy(t);
 	}
 	cddb_quit();
 }
@@ -860,8 +859,9 @@ static void drive_set_volume(int l, int r)
 }
 
 
-#if defined(CDIOCREADAUDIO)
 #ifdef __FreeBSD__
+# if defined(CDIOCREADAUDIO)
+/* Digital extraction before ATAng */
 int read_audio_data(int fd, int pos, int num, void *buf)
 {
 	struct ioc_read_audio cdra;
@@ -876,10 +876,20 @@ int read_audio_data(int fd, int pos, int num, void *buf)
 
 	return cdra.nframes;
 }
-#else
-#error Please test on other <sys/cdio.h> platforms.
+# else
+/* Digital extraction with ATAng */
+int read_audio_data(int fd, int pos, int num, void *buf)
+{	
+	int bsize = 2352;
+
+	if (ioctl(fd, CDRIOCSETBLOCKSIZE, &bsize) == -1)
+		return -errno;
+	if (pread(fd, buf, num*bsize, (pos - 150)*bsize) != num*bsize)
+		return 0;
+	return num;	 	
+}
+# endif /* CDIOCREADAUDIO */
 #endif
-#endif /* CDIOCREADAUDIO */
 
 #ifdef XMMS_CDROM_BSD_NETBSD /* NetBSD, OpenBSD */
 
@@ -1152,7 +1162,10 @@ static void oss_set_volume(int l, int r, int mixer_line)
 static void get_volume(int *l, int *r)
 {
 	if (cdda_playing.drive.dae)
-		cdda_ip.output->get_volume(l, r);
+	{
+		if (cdda_ip.output->get_volume)
+			cdda_ip.output->get_volume(l, r);
+	}
 	else if (cdda_playing.drive.mixer == CDDA_MIXER_OSS)
 		oss_get_volume(l, r, cdda_playing.drive.oss_mixer);
 	else if(cdda_playing.drive.mixer == CDDA_MIXER_DRIVE)
