@@ -409,29 +409,15 @@ static void *play_loop(void *arg)
 	pthread_exit(NULL);
 }
 
-static void
-     configure()
+static void configure()
 {
-	GtkWidget *notebook1;
-	GtkWidget *vbox;
-	GtkWidget *vbox1;
-	GtkWidget *hbox1;
-	GtkWidget *Resolution_Frame;
-	GtkWidget *vbox4;
-	GSList *resolution_group = NULL;
-	GtkWidget *Channels_Frame;
-	GtkWidget *vbox5;
-	GSList *vbox5_group = NULL;
-	GtkWidget *Downsample_Frame;
-	GtkWidget *vbox3;
+	GtkWidget *notebook1, *vbox, *vbox1, *hbox1, *Resolution_Frame, *vbox4;
+	GSList *resolution_group = NULL, *vbox5_group = NULL;
+	GtkWidget *Channels_Frame, *vbox5, *Downsample_Frame, *vbox3;
 	GSList *sample_group = NULL;
-	GtkWidget *vbox6;
-	GtkWidget *Quality_Label;
-	GtkWidget *Options_Label;
+	GtkWidget *vbox6, *Quality_Label, *Options_Label;
 	GtkWidget *pansep_label, *pansep_hscale;
-	GtkWidget *bbox;
-	GtkWidget *ok;
-	GtkWidget *cancel;
+	GtkWidget *bbox, *ok, *cancel;
 
 	if (!mikmod_conf_window)
 	{
@@ -515,7 +501,7 @@ static void
 		if (mikmod_cfg.force_mono == 1)
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Chan_MO), TRUE);
 
-		Downsample_Frame = gtk_frame_new(_("Down sample:"));
+		Downsample_Frame = gtk_frame_new(_("Downsample:"));
 		gtk_object_set_data(GTK_OBJECT(mikmod_conf_window), "Downsample_Frame", Downsample_Frame);
 		gtk_widget_show(Downsample_Frame);
 		gtk_box_pack_start(GTK_BOX(vbox1), Downsample_Frame, TRUE, TRUE, 0);
