@@ -80,6 +80,7 @@ void mpg123_stream_close(void)
 		mpg123_http_close();
 }
 
+#if 0
 /**************************************** 
  * HACK,HACK,HACK: step back <num> frames 
  * can only work if the 'stream' isn't a real stream but a file
@@ -90,6 +91,7 @@ static int stream_back_bytes(int bytes)
 		return -1;
 	return 0;
 }
+#endif
 
 static int stream_head_read(unsigned long *newhead)
 {
@@ -136,10 +138,12 @@ static long stream_tell(void)
 	return ftell(filept);
 }
 
+#if 0
 static void stream_rewind(void)
 {
 	fseek(filept, 0, SEEK_SET);
 }
+#endif
 
 int mpg123_stream_jump_to_frame(struct frame *fr, int frame)
 {
