@@ -616,28 +616,27 @@ static void *http_buffer_loop(void *arg)
 	
 	if (mpg123_cfg.save_http_stream)
 	{
-		char *output_name;
+		char *output_name, *fname, *temp;
 		int i = 1;
 
 		file = mpg123_http_get_title(url);
-		output_name = file;
-		if (!strncasecmp(output_name, "http://", 7))
-			output_name += 7;
-		temp = strrchr(output_name, '.');
+		fname = file;
+		if (!strncasecmp(fname, "http://", 7))
+			fname += 7;
+		temp = strrchr(fname, '.');
 		if (temp && !strcasecmp(temp, ".mp3"))
 			*temp = '\0';
 
-		while ((temp = strchr(output_name, '/')))
+		while ((temp = strchr(fname, '/')))
 			*temp = '_';
 		output_name = g_strdup_printf("%s/%s.mp3",
-					      mpg123_cfg.save_http_path,
-					      output_name);
+					      mpg123_cfg.save_http_path, fname);
 		while (!access(output_name, F_OK) && i < 100000)
 		{
 			g_free(output_name);
 			output_name = g_strdup_printf("%s/%s-%d.mp3",
 						      mpg123_cfg.save_http_path,
-						      output_name, i++);
+						      fname, i++);
 		}
 
 		g_free(file);
@@ -774,7 +773,8 @@ static int udp_establish_listener(int *sock)
 #endif
 	{
 		g_log(NULL, G_LOG_LEVEL_CRITICAL,
-		      "udp_establish_listener(): unable to create socket");
+		      "udp_establish_listener(): unable to create socket: %s",
+		      strerror(errno));
 		return -1;
 	}
 
