@@ -871,6 +871,13 @@ static void *decode_loop(void *arg)
 			{
 				mpg123_info->num_frames = xing_header.frames;
 				have_xing_header = TRUE;
+				if (xing_header.bytes == 0)
+				{
+					if (mpg123_info->filesize > 0)
+						xing_header.bytes = mpg123_info->filesize;
+					else
+						have_xing_header = FALSE;
+				}
 				mpg123_read_frame(&fr);
 			}
 		}
