@@ -4,6 +4,7 @@
 #include <string.h>
 #include <glib.h>
 #include <gtk/gtk.h>
+#include <pthread.h>
 
 #include "vorbis.h"
 #include "libxmms/configfile.h"
@@ -13,28 +14,33 @@
 #include "libxmms/util.h"
 #include "xmms/plugin.h"
 
+extern pthread_mutex_t vf_mutex;
+
 static GtkWidget *vorbis_configurewin = NULL;
 static GtkWidget *vbox, *notebook;
 
-static GtkObject *streaming_size_adj, *streaming_pre_adj;
 static GtkWidget *streaming_proxy_use, *streaming_proxy_host_entry;
-static GtkWidget *streaming_proxy_port_entry, *streaming_save_use, *streaming_save_entry;
+static GtkWidget *streaming_proxy_port_entry, *streaming_save_entry;
+static GtkWidget *streaming_save_use, *streaming_size_spin, *streaming_pre_spin;
 static GtkWidget *streaming_proxy_auth_use;
 static GtkWidget *streaming_proxy_auth_pass_entry, *streaming_proxy_auth_user_entry;
 static GtkWidget *streaming_proxy_auth_user_label, *streaming_proxy_auth_pass_label;
-static GtkWidget *streaming_proxy_hbox, *streaming_proxy_auth_hbox, *streaming_save_dirbrowser;
-static GtkWidget *streaming_save_hbox;
+static GtkWidget *streaming_proxy_hbox, *streaming_proxy_auth_hbox;
+static GtkWidget *streaming_save_dirbrowser, *streaming_save_hbox;
 static GtkWidget *title_tag_override, *title_tag_box, *title_tag_entry, *title_desc;
+static GtkWidget *rg_switch, *rg_clip_switch, *rg_booster_switch, *rg_track_gain;
 
 vorbis_config_t vorbis_cfg;
 
 static void vorbis_configurewin_ok(GtkWidget * widget, gpointer data)
 {
 	ConfigFile *cfg;
-	gchar *filename;
+	GtkToggleButton *tb;
 
-	vorbis_cfg.http_buffer_size = (gint)GTK_ADJUSTMENT(streaming_size_adj)->value;
-	vorbis_cfg.http_prebuffer = (gint)GTK_ADJUSTMENT(streaming_pre_adj)->value;
+	vorbis_cfg.http_buffer_size =
+		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(streaming_size_spin));
+	vorbis_cfg.http_prebuffer = 
+		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(streaming_pre_spin));
 
 	vorbis_cfg.use_proxy = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(streaming_proxy_use));
 	g_free(vorbis_cfg.proxy_host);
@@ -60,31 +66,59 @@ static void vorbis_configurewin_ok(GtkWidget * widget, gpointer data)
 	g_free(vorbis_cfg.tag_format);
         vorbis_cfg.tag_format = g_strdup(gtk_entry_get_text(GTK_ENTRY(title_tag_entry)));	
 
-	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
-	cfg = xmms_cfg_open_file(filename);
-	if (!cfg)
-		cfg = xmms_cfg_new();
-	xmms_cfg_write_int(cfg, "vorbis", "http_buffer_size", vorbis_cfg.http_buffer_size);
-	xmms_cfg_write_int(cfg, "vorbis", "http_prebuffer", vorbis_cfg.http_prebuffer);
-	xmms_cfg_write_boolean(cfg, "vorbis", "use_proxy", vorbis_cfg.use_proxy);
-	xmms_cfg_write_string(cfg, "vorbis", "proxy_host", vorbis_cfg.proxy_host);
-	xmms_cfg_write_int(cfg, "vorbis", "proxy_port", vorbis_cfg.proxy_port);
-	xmms_cfg_write_boolean(cfg, "vorbis", "proxy_use_auth", vorbis_cfg.proxy_use_auth);
-	if(vorbis_cfg.proxy_user)
-		xmms_cfg_write_string(cfg, "vorbis", "proxy_user", vorbis_cfg.proxy_user);
+	tb = GTK_TOGGLE_BUTTON(title_tag_override);
+	vorbis_cfg.tag_override = gtk_toggle_button_get_active(tb);
+	tb = GTK_TOGGLE_BUTTON(rg_switch);
+	vorbis_cfg.use_replaygain = gtk_toggle_button_get_active(tb);
+	tb = GTK_TOGGLE_BUTTON(rg_clip_switch);
+	vorbis_cfg.use_anticlip = gtk_toggle_button_get_active(tb);
+	tb = GTK_TOGGLE_BUTTON(rg_booster_switch);
+	vorbis_cfg.use_booster = gtk_toggle_button_get_active(tb);
+	tb = GTK_TOGGLE_BUTTON(rg_track_gain);
+	if (gtk_toggle_button_get_active(tb))
+		vorbis_cfg.replaygain_mode = REPLAYGAIN_MODE_TRACK;
+	else
+		vorbis_cfg.replaygain_mode = REPLAYGAIN_MODE_ALBUM;
+
+	
+	cfg = xmms_cfg_open_default_file();
+
+	xmms_cfg_write_int(cfg, "vorbis", "http_buffer_size",
+			   vorbis_cfg.http_buffer_size);
+	xmms_cfg_write_int(cfg, "vorbis", "http_prebuffer",
+			   vorbis_cfg.http_prebuffer);
+	xmms_cfg_write_boolean(cfg, "vorbis", "use_proxy",
+			       vorbis_cfg.use_proxy);
+	xmms_cfg_write_string(cfg, "vorbis", "proxy_host",
+			      vorbis_cfg.proxy_host);
+	xmms_cfg_write_boolean(cfg, "vorbis", "save_http_stream",
+			       vorbis_cfg.save_http_stream);
+	xmms_cfg_write_string(cfg, "vorbis", "save_http_path",
+			      vorbis_cfg.save_http_path);
+	xmms_cfg_write_boolean(cfg, "vorbis", "tag_override",
+			       vorbis_cfg.tag_override);
+	xmms_cfg_write_string(cfg, "vorbis", "tag_format",
+			      vorbis_cfg.tag_format);
+	xmms_cfg_write_int(cfg, "vorbis", "proxy_port",
+			   vorbis_cfg.proxy_port);
+	xmms_cfg_write_boolean(cfg, "vorbis", "proxy_use_auth",
+			       vorbis_cfg.proxy_use_auth);
+	if (vorbis_cfg.proxy_user)
+		xmms_cfg_write_string(cfg, "vorbis", "proxy_user",
+				      vorbis_cfg.proxy_user);
 	else
 		xmms_cfg_remove_key(cfg, "vorbis", "proxy_user");
-	if(vorbis_cfg.proxy_pass)
-		xmms_cfg_write_string(cfg, "vorbis", "proxy_pass", vorbis_cfg.proxy_pass);
+	if (vorbis_cfg.proxy_pass)
+		xmms_cfg_write_string(cfg, "vorbis", "proxy_pass",
+				      vorbis_cfg.proxy_pass);
 	else
 		xmms_cfg_remove_key(cfg, "vorbis", "proxy_pass");
-	xmms_cfg_write_boolean(cfg, "vorbis", "save_http_stream", vorbis_cfg.save_http_stream);
-	xmms_cfg_write_string(cfg, "vorbis", "save_http_path", vorbis_cfg.save_http_path);
-	xmms_cfg_write_boolean(cfg, "vorbis", "tag_override", vorbis_cfg.tag_override);
-	xmms_cfg_write_string(cfg, "vorbis", "tag_format", vorbis_cfg.tag_format);
-	xmms_cfg_write_file(cfg, filename);
+	xmms_cfg_write_boolean(cfg, "vorbis", "use_anticlip", vorbis_cfg.use_anticlip);
+	xmms_cfg_write_boolean(cfg, "vorbis", "use_replaygain", vorbis_cfg.use_replaygain);
+	xmms_cfg_write_int(cfg, "vorbis", "replaygain_mode", vorbis_cfg.replaygain_mode);
+	xmms_cfg_write_boolean(cfg, "vorbis", "use_booster", vorbis_cfg.use_booster);
+	xmms_cfg_write_default_file(cfg);
 	xmms_cfg_free(cfg);
-	g_free(filename);
 	gtk_widget_destroy(vorbis_configurewin);
 }
 
@@ -117,14 +151,21 @@ static void streaming_save_dirbrowser_cb(gchar * dir)
 
 static void streaming_save_browse_cb(GtkWidget * w, gpointer data)
 {
-	if (!streaming_save_dirbrowser)
-	{
-		streaming_save_dirbrowser = xmms_create_dir_browser(_("Select the directory where you want to store the Ogg Vorbis streams:"),
-								    vorbis_cfg.save_http_path, GTK_SELECTION_SINGLE, streaming_save_dirbrowser_cb);
-		gtk_signal_connect(GTK_OBJECT(streaming_save_dirbrowser), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &streaming_save_dirbrowser);
-		gtk_window_set_transient_for(GTK_WINDOW(streaming_save_dirbrowser), GTK_WINDOW(vorbis_configurewin));
-		gtk_widget_show(streaming_save_dirbrowser);
-	}
+	if (streaming_save_dirbrowser)
+		return;
+
+	streaming_save_dirbrowser =
+		xmms_create_dir_browser(_("Select the directory where you want "
+					  "to store the Ogg Vorbis streams:"),
+					vorbis_cfg.save_http_path,
+					GTK_SELECTION_SINGLE,
+					streaming_save_dirbrowser_cb);
+	gtk_signal_connect(GTK_OBJECT(streaming_save_dirbrowser),
+			   "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed),
+			   &streaming_save_dirbrowser);
+	gtk_window_set_transient_for(GTK_WINDOW(streaming_save_dirbrowser),
+				     GTK_WINDOW(vorbis_configurewin));
+	gtk_widget_show(streaming_save_dirbrowser);
 }
 
 static void streaming_save_use_cb(GtkWidget * w, gpointer data)
@@ -138,37 +179,47 @@ static void streaming_save_use_cb(GtkWidget * w, gpointer data)
 
 static void configure_destroy(GtkWidget * w, gpointer data)
 {
-	if (streaming_save_dirbrowser)
-		gtk_widget_destroy(streaming_save_dirbrowser);
+/*  	if (streaming_save_dirbrowser) */
+/*  		gtk_widget_destroy(streaming_save_dirbrowser); */
 }
 
 static void title_tag_override_cb(GtkWidget * w, gpointer data)
 {
-	vorbis_cfg.tag_override = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(title_tag_override));
+	gboolean override;
+	override = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(title_tag_override));
+	gtk_widget_set_sensitive(title_tag_box, override);
+	gtk_widget_set_sensitive(title_desc, override);
+}
 
-	gtk_widget_set_sensitive(title_tag_box, vorbis_cfg.tag_override);
-	gtk_widget_set_sensitive(title_desc, vorbis_cfg.tag_override);
+static void rg_switch_cb(GtkWidget * w, gpointer data)
+{
+	gtk_widget_set_sensitive(GTK_WIDGET(data), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
 }
 
 void vorbis_configure(void)
 {
 	GtkWidget *streaming_vbox;
 	GtkWidget *streaming_buf_frame, *streaming_buf_hbox;
-	GtkWidget *streaming_size_box, *streaming_size_label, *streaming_size_spin;
-	GtkWidget *streaming_pre_box, *streaming_pre_label, *streaming_pre_spin;
+	GtkWidget *streaming_size_box, *streaming_size_label;
+	GtkWidget *streaming_pre_box, *streaming_pre_label;
 	GtkWidget *streaming_proxy_frame, *streaming_proxy_vbox;
 	GtkWidget *streaming_proxy_port_label, 	*streaming_proxy_host_label;
 	GtkWidget *streaming_save_frame, *streaming_save_vbox;
 	GtkWidget *streaming_save_label, *streaming_save_browse;
 	GtkWidget *title_frame, *title_tag_vbox, *title_tag_label;
+	GtkWidget *rg_frame, *rg_vbox;
 	GtkWidget *bbox, *ok, *cancel;
+	GtkWidget *rg_type_frame, *rg_type_vbox, *rg_album_gain;
+	GtkObject *streaming_size_adj, *streaming_pre_adj;
 
-	gchar *temp;
+	char *temp;
 
-	if (vorbis_configurewin != NULL) {
+	if (vorbis_configurewin != NULL)
+	{
 		gdk_window_raise(vorbis_configurewin->window);
 		return;
 	}
+
 	vorbis_configurewin = gtk_window_new(GTK_WINDOW_DIALOG);
 	gtk_signal_connect(GTK_OBJECT(vorbis_configurewin), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &vorbis_configurewin);
 	gtk_signal_connect(GTK_OBJECT(vorbis_configurewin), "destroy", GTK_SIGNAL_FUNC(configure_destroy), &vorbis_configurewin);
@@ -339,6 +390,57 @@ void vorbis_configure(void)
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), title_frame, gtk_label_new(_("Title")));
 
+	/* Replay Gain.. */
+	
+	rg_frame = gtk_frame_new(_("ReplayGain Settings:"));
+	gtk_container_border_width(GTK_CONTAINER(rg_frame), 5);
+	
+	rg_vbox = gtk_vbox_new(FALSE, 10);
+	gtk_container_border_width(GTK_CONTAINER(rg_vbox), 5);
+	gtk_container_add(GTK_CONTAINER(rg_frame), rg_vbox);
+	
+	rg_clip_switch = gtk_check_button_new_with_label(_("Enable Clipping Prevention"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rg_clip_switch), vorbis_cfg.use_anticlip);
+	gtk_box_pack_start(GTK_BOX(rg_vbox), rg_clip_switch, FALSE, FALSE, 0);
+
+	rg_switch = gtk_check_button_new_with_label(_("Enable ReplayGain"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rg_switch), vorbis_cfg.use_replaygain);
+	gtk_box_pack_start(GTK_BOX(rg_vbox), rg_switch, FALSE, FALSE, 0);
+	
+	rg_type_frame = gtk_frame_new(_("ReplayGain Type:"));
+	gtk_box_pack_start(GTK_BOX(rg_vbox), rg_type_frame, FALSE, FALSE, 0);
+
+	gtk_signal_connect(GTK_OBJECT(rg_switch), "toggled", rg_switch_cb, rg_type_frame);
+	
+	rg_type_vbox = gtk_vbox_new(FALSE, 5);
+	gtk_container_set_border_width(GTK_CONTAINER(rg_type_vbox), 5);
+	gtk_container_add(GTK_CONTAINER(rg_type_frame), rg_type_vbox);
+
+	rg_track_gain = gtk_radio_button_new_with_label(NULL, _("use Track Gain/Peak"));
+	if (vorbis_cfg.replaygain_mode == REPLAYGAIN_MODE_TRACK)
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rg_track_gain), TRUE);
+	else
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rg_track_gain), FALSE);
+	gtk_box_pack_start(GTK_BOX(rg_type_vbox), rg_track_gain, FALSE, FALSE, 0);
+
+	rg_album_gain = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(rg_track_gain)), _("use Album Gain/Peak"));
+	if (vorbis_cfg.replaygain_mode == REPLAYGAIN_MODE_ALBUM)
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rg_album_gain), TRUE);
+	else
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rg_album_gain), FALSE);
+	gtk_box_pack_start(GTK_BOX(rg_type_vbox), rg_album_gain, FALSE, FALSE, 0);
+
+	if (!vorbis_cfg.use_replaygain)
+		gtk_widget_set_sensitive(rg_type_frame, FALSE);
+
+	rg_booster_switch = gtk_check_button_new_with_label(_("Enable 6dB Boost + Hard Limiting"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rg_booster_switch), vorbis_cfg.use_booster);
+	gtk_box_pack_start(GTK_BOX(rg_vbox), rg_booster_switch, FALSE, FALSE, 0);
+
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), rg_frame, gtk_label_new(_("ReplayGain")));
+
+	/* Buttons */
+
 	bbox = gtk_hbutton_box_new();
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
 	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
