@@ -85,17 +85,17 @@ InputPlugin vorbis_ip =
 {
 	NULL,
 	NULL,
-	NULL,      /* description */
-	vorbis_init,	/* init */
-	vorbis_aboutbox,   /* aboutbox */
-	vorbis_configure,  /* configure */
+	NULL,               /* description */
+	vorbis_init,        /* init */
+	vorbis_aboutbox,    /* aboutbox */
+	vorbis_configure,   /* configure */
 	vorbis_check_file,  /* is_our_file */
 	NULL,
 	vorbis_play,
 	vorbis_stop,
 	vorbis_pause,
 	vorbis_seek,
-	NULL,       /* set eq */
+	NULL,               /* set eq */
 	vorbis_time,
 	NULL,
 	NULL,
@@ -224,14 +224,17 @@ static int vorbis_process_data(int last_section, gboolean use_rg, float rg_scale
 	int current_section;
 
 	pthread_mutex_lock(&vf_mutex);
-	if (use_rg) {
-		bytes = ov_read_float(&vf, &pcm, sizeof(pcmout)/2/channels, &current_section);
+	if (use_rg)
+	{
+		bytes = ov_read_float(&vf, &pcm, sizeof(pcmout)/2/channels,
+				      &current_section);
 		if (bytes > 0)
 			bytes = vorbis_process_replaygain(pcm, bytes, channels,
 			                                  pcmout, rg_scale);
-	} else {
-		bytes = ov_read(&vf, pcmout, sizeof(pcmout), bep, 2, 1, &current_section);
 	}
+	else
+		bytes = ov_read(&vf, pcmout, sizeof(pcmout), bep, 2, 1,
+				&current_section);
 
 	switch (bytes)
 	{
@@ -585,13 +588,15 @@ static gboolean vorbis_update_replaygain(float *scale)
 		{
 			rg_gain = vorbis_comment_query(comment, "replaygain_album_gain", 0);
 			if (!rg_gain)
-				rg_gain = vorbis_comment_query(comment, "rg_audiophile", 0); /* Old */
+				/* Obsolete name */
+				rg_gain = vorbis_comment_query(comment, "rg_audiophile", 0);
 		}
 
 		if (!rg_gain)
 			rg_gain = vorbis_comment_query(comment, "replaygain_track_gain", 0);
 		if (!rg_gain)
-			rg_gain = vorbis_comment_query(comment, "rg_radio", 0); /* Old */
+			/* Obsolete name */
+			rg_gain = vorbis_comment_query(comment, "rg_radio", 0);
 		
 		/* FIXME: Make sure this string is the correct format first? */
 		if (rg_gain)
@@ -601,12 +606,17 @@ static gboolean vorbis_update_replaygain(float *scale)
 	if (vorbis_cfg.use_anticlip)
 	{
 		if (vorbis_cfg.replaygain_mode == REPLAYGAIN_MODE_ALBUM)
-			rg_peak_str = vorbis_comment_query(comment, "replaygain_album_peak", 0);
+			rg_peak_str =
+				vorbis_comment_query(comment,
+						     "replaygain_album_peak", 0);
 
 		if (!rg_peak_str)
-			rg_peak_str = vorbis_comment_query(comment, "replaygain_track_peak", 0);
+			rg_peak_str =
+				vorbis_comment_query(comment,
+						     "replaygain_track_peak", 0);
 		if (!rg_peak_str)
-			rg_peak_str = vorbis_comment_query(comment, "rg_peak", 0); /* Old */
+			/* Obsolete name */
+			rg_peak_str = vorbis_comment_query(comment, "rg_peak", 0);
 
 		if (rg_peak_str)
 			rg_peak = atof(rg_peak_str);
@@ -618,7 +628,6 @@ static gboolean vorbis_update_replaygain(float *scale)
 	}
 	
 	if (*scale != 1.0 || vorbis_cfg.use_booster) {
-		/* safety */
 		if (*scale > 15.0)
 			*scale = 15.0;
 
@@ -662,10 +671,10 @@ static long vorbis_process_replaygain(float **pcm, int samples, int ch, char *pc
 	return 2 * ch * samples;
 }			
 
-static gchar *vorbis_generate_title(OggVorbis_File *vorbisfile, char *fn)
+static char *vorbis_generate_title(OggVorbis_File *vorbisfile, char *fn)
 {
 	/* Caller should hold vf_mutex */
-	gchar *displaytitle = NULL, *tmp, *path, *temp;
+	char *displaytitle = NULL, *tmp, *path, *temp;
 	vorbis_comment *comment;
 	TitleInput *input;
 
@@ -686,6 +695,8 @@ static gchar *vorbis_generate_title(OggVorbis_File *vorbisfile, char *fn)
 		if ((tmp = vorbis_comment_query(comment, "tracknumber", 0)) != NULL)
 			input->track_number = atoi(tmp);
 		input->date = convert_from_utf8(vorbis_comment_query(comment, "date", 0));
+		if ((tmp = vorbis_comment_query(comment, "year", 0)) != NULL)
+			input->year = atoi(tmp);
 		input->genre = convert_from_utf8(vorbis_comment_query(comment, "genre", 0));
 		/* "comment" = user comment, not "" */
 		input->comment = convert_from_utf8(vorbis_comment_query(comment, "comment", 0));
@@ -706,7 +717,12 @@ static gchar *vorbis_generate_title(OggVorbis_File *vorbisfile, char *fn)
 	if (!displaytitle)
 	{
 		if (!vorbis_is_streaming)
-			displaytitle = g_strdup_printf("%s", g_basename(fn));
+		{
+			char *tmp;
+			displaytitle = g_strdup(g_basename(fn));
+			if ((tmp = strrchr(displaytitle, '.')) != NULL)
+				*tmp = '\0';
+		}
 		else
 			displaytitle = vorbis_http_get_title(fn);
 	}
