@@ -2,7 +2,7 @@
  *  Copyright (C) 2001-2003 Matthieu Sozeau <mattam@altern.org>
  *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2003  Haavard Kvaalen
+ *  Copyright (C) 1999-2004  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,8 +29,6 @@ static snd_pcm_channel_area_t   *areas              = NULL;
 
 static snd_output_t		*logs 		    = NULL;
 
-static snd_pcm_uframes_t         alsa_period_size;
-/* Hmm, this is currently bytes per ms */
 static int 			 alsa_bps 	    = 0;
 static guint64			 alsa_total_written = 0;
 
@@ -38,13 +36,9 @@ static guint64			 alsa_total_written = 0;
 static snd_mixer_elem_t 	*pcm_element 	    = NULL;
 static snd_mixer_t 		*mixer 		    = NULL;
 
-static int                       mmap = 1;
-/* Loop */
-static int 			 going              = 0;
-static gboolean 		 paused 	    = FALSE;
-static int                       first_write        = 1;
+static gboolean mmap, force_start, going, paused;
 
-static gpointer 		 buffer;
+static gpointer buffer;
 
 static int alsa_can_pause;
 
@@ -59,7 +53,6 @@ static struct snd_format *inputf = NULL;
 static struct snd_format *effectf = NULL;
 static struct snd_format *outputf = NULL;
 
-
 static int alsa_setup(struct snd_format *f);
 static void alsa_mmap_audio(char *data, int length);
 static void alsa_write_audio(gpointer data, int length);
@@ -72,7 +65,7 @@ static convert_func_t alsa_convert_func;
 static convert_channel_func_t alsa_stereo_convert_func;
 static convert_freq_func_t alsa_frequency_convert_func;
 
-static struct {
+static const struct {
 	AFormat xmms;
 	snd_pcm_format_t alsa;
 } format_table[] =
@@ -99,6 +92,8 @@ static struct {
 };
 
 
+static void debug(char *str, ...) G_GNUC_PRINTF(1, 2);
+
 static void debug(char *str, ...)
 {
 	va_list args;
@@ -139,8 +134,6 @@ static void xrun_recover(void)
 	{
 		if ((err = snd_pcm_prepare(alsa_pcm)) < 0)
 			g_warning("xrun_recover(): snd_pcm_prepare() failed.");
-		if (mmap)
-			first_write = 1;
 	}
 }
 
@@ -168,38 +161,45 @@ static snd_pcm_sframes_t alsa_get_avail(void)
 
 int alsa_free(void)
 {
-	snd_pcm_sframes_t avail;
-
 	if (paused)
 		return 0;
 	else
 	{
-		avail = alsa_get_avail();
-		return snd_pcm_frames_to_bytes(alsa_pcm, avail);
+		int err;
+		if (force_start &&
+		    snd_pcm_state(alsa_pcm) == SND_PCM_STATE_PREPARED)
+		{
+			if ((err = snd_pcm_start(alsa_pcm)) < 0)
+				g_warning("alsa_free(): snd_pcm_start() "
+					  "failed: %s", snd_strerror(-err));
+			else
+				debug("Stream started");
+		}
+		force_start = TRUE;
+
+		return snd_pcm_frames_to_bytes(alsa_pcm, alsa_get_avail());
 	}
 }
 
 void alsa_pause(short p)
 {
 	debug("alsa_pause");
-	paused = p;
+	if (p)
+		paused = TRUE;
 
-	if (alsa_can_pause && !mmap)
+	if (alsa_can_pause)
 		snd_pcm_pause(alsa_pcm, p);
-	else
-	{
-	    if (p)
-		    snd_pcm_drop(alsa_pcm);
-	    snd_pcm_prepare(alsa_pcm);
-	}
+	else if (p)
+		snd_pcm_drop(alsa_pcm);
 
-	first_write = 1;
+	if (!p)
+		paused = FALSE;
 }
 
-
 void alsa_close(void)
 {
 	int err, started;
+
 	debug("Closing device");
 
 	started = going;
@@ -217,12 +217,12 @@ void alsa_close(void)
 	{
 		if (started)
 			if ((err = snd_pcm_drop(alsa_pcm)) < 0)
-				g_warning ("alsa_pcm_drop() failed: %s",
-					   snd_strerror(-err));
+				g_warning("alsa_pcm_drop() failed: %s",
+					  snd_strerror(-err));
 
 		if ((err = snd_pcm_close(alsa_pcm)) < 0)
-			g_warning ("alsa_pcm_close() failed: %s",
-				   snd_strerror(-err));
+			g_warning("alsa_pcm_close() failed: %s",
+				  snd_strerror(-err));
 		alsa_pcm = NULL;
 	}
 
@@ -241,13 +241,14 @@ void alsa_close(void)
 	g_free(effectf);
 	effectf = NULL;
 
+	alsa_save_config();
+
 	debug("Device closed");
 }
 
 static void alsa_reopen(struct snd_format *f)
 {
 	unsigned int tmp = alsa_get_written_time();
-	first_write = 1;
 
 	if (alsa_pcm != NULL)
 	{
@@ -272,7 +273,7 @@ static void alsa_reopen(struct snd_format *f)
 
 void alsa_flush(int time)
 {
-	alsa_total_written = (guint64) time * alsa_bps;
+	alsa_total_written = (guint64) time * alsa_bps / 1000;
 }
 
 static void parse_mixer_name(char *str, char **name, int *index)
@@ -295,53 +296,80 @@ static void parse_mixer_name(char *str, char **name, int *index)
 	}
 }
 
-int alsa_setup_mixer(void)
+int alsa_get_mixer(snd_mixer_t **mixer, int card) 
 {
-	char *dev, *name;
-	long int a, b;
-	snd_mixer_selem_id_t *selem_id;
-	long alsa_min_vol, alsa_max_vol;
-	int err, index;
+	char *dev;
+	int err;
+	
+	debug("alsa_get_mixer");
+	
+	dev = g_strdup_printf("hw:%i", card);
 
-	debug("alsa_setup_mixer");
-
-	dev = g_strdup_printf("hw:%i", alsa_cfg.mixer_card);
-
-	if ((err = snd_mixer_open(&mixer, 0)) < 0)
+	if ((err = snd_mixer_open(mixer, 0)) < 0)
 	{
-		g_warning("alsa_setup_mixer(): Failed to open empty mixer: %s",
+		g_warning("alsa_get_mixer(): Failed to open empty mixer: %s",
 			  snd_strerror(-err));
 		mixer = NULL;
 		return -1;
 	}
-	if ((err = snd_mixer_attach(mixer, dev)) < 0)
+	if ((err = snd_mixer_attach(*mixer, dev)) < 0)
 	{
-		g_warning("alsa_setup_mixer(): Attaching to mixer %s failed: %s",
+		g_warning("alsa_get_mixer(): Attaching to mixer %s failed: %s",
 			  dev, snd_strerror(-err));
 		return -1;
 	}
-	if ((err = snd_mixer_selem_register(mixer, NULL, NULL)) < 0)
+	if ((err = snd_mixer_selem_register(*mixer, NULL, NULL)) < 0)
 	{
-		g_warning("alsa_setup_mixer(): Failed to register mixer: %s",
+		g_warning("alsa_get_mixer(): Failed to register mixer: %s",
 			  snd_strerror(-err));
 		return -1;
 	}
-	if ((err = snd_mixer_load(mixer)) < 0)
+	if ((err = snd_mixer_load(*mixer)) < 0)
 	{
-		g_warning("alsa_setup_mixer(): Failed to load mixer: %s",
+		g_warning("alsa_get_mixer(): Failed to load mixer: %s",
 			  snd_strerror(-err));
 		return -1;
 	}
+	
+	g_free(dev);
 
+	return (*mixer != NULL);
+}
+	
+
+snd_mixer_elem_t* alsa_get_mixer_elem(snd_mixer_t *mixer, char *name, int index)
+{
+	snd_mixer_selem_id_t *selem_id;
+	snd_mixer_elem_t* elem;
 	snd_mixer_selem_id_alloca(&selem_id);
 
+	if (index != -1)
+		snd_mixer_selem_id_set_index(selem_id, index);
+	if (name != NULL)
+		snd_mixer_selem_id_set_name(selem_id, name);
+
+	elem = snd_mixer_find_selem(mixer, selem_id);
+	
+	return elem;
+}
+
+int alsa_setup_mixer(void)
+{
+	char *name;
+	long int a, b;
+	long alsa_min_vol, alsa_max_vol;
+	int err, index;
+
+	debug("alsa_setup_mixer");
+	
+	if ((err = alsa_get_mixer(&mixer, alsa_cfg.mixer_card)) < 0)
+		return err;
+
 	parse_mixer_name(alsa_cfg.mixer_device, &name, &index);
-
-	snd_mixer_selem_id_set_index(selem_id, index);
-	snd_mixer_selem_id_set_name(selem_id, name);
+	
+	pcm_element = alsa_get_mixer_elem(mixer, name, index);
+	
 	g_free(name);
-
-	pcm_element = snd_mixer_find_selem(mixer, selem_id);
 	
 	if (!pcm_element)
 	{
@@ -350,6 +378,16 @@ int alsa_setup_mixer(void)
 		return -1;
 	}
 
+	/*
+	 * Work around a bug in alsa-lib up to 1.0.0rc2 where the
+	 * new range don't take effect until the volume is changed.
+	 * This hack should be removed once we depend on Alsa 1.0.0.
+	 */
+	snd_mixer_selem_get_playback_volume(pcm_element,
+					    SND_MIXER_SCHN_FRONT_LEFT, &a);
+	snd_mixer_selem_get_playback_volume(pcm_element,
+					    SND_MIXER_SCHN_FRONT_RIGHT, &b);
+
 	snd_mixer_selem_get_playback_volume_range(pcm_element,
 						  &alsa_min_vol, &alsa_max_vol);
 	snd_mixer_selem_set_playback_volume_range(pcm_element, 0, 100);
@@ -360,14 +398,9 @@ int alsa_setup_mixer(void)
 		return -1;
 	}
 
-	snd_mixer_selem_get_playback_volume(pcm_element,
-					    SND_MIXER_SCHN_FRONT_LEFT, &a);
-	snd_mixer_selem_get_playback_volume(pcm_element,
-					    SND_MIXER_SCHN_FRONT_RIGHT, &b);
+	if (!alsa_cfg.soft_volume)
+		alsa_set_volume(a * 100 / alsa_max_vol, b * 100 / alsa_max_vol);
 
-	alsa_set_volume(a * 100 / alsa_max_vol, b * 100 / alsa_max_vol);
-
-	g_free(dev);
 	debug("alsa_setup_mixer: end");
 
 	return 0;
@@ -376,6 +409,7 @@ int alsa_setup_mixer(void)
 void alsa_get_volume(int *l, int *r)
 {
 	static gboolean first = TRUE;
+	long ll = *l, lr = *r;
 
 	if (first)
 	{
@@ -386,28 +420,44 @@ void alsa_get_volume(int *l, int *r)
 	if (!pcm_element)
 		return;
 
-	snd_mixer_handle_events (mixer);
-
-	snd_mixer_selem_get_playback_volume(pcm_element,
-					    SND_MIXER_SCHN_FRONT_LEFT,
-					    (long int *) l);
-	snd_mixer_selem_get_playback_volume(pcm_element,
-					    SND_MIXER_SCHN_FRONT_RIGHT,
-					    (long int *) r);
+	snd_mixer_handle_events(mixer);
 
+	if (alsa_cfg.soft_volume)
+	{
+		*l = alsa_cfg.vol.left;
+		*r = alsa_cfg.vol.right;
+	}
+	else
+	{
+		snd_mixer_selem_get_playback_volume(pcm_element,
+						    SND_MIXER_SCHN_FRONT_LEFT,
+						    &ll);
+		snd_mixer_selem_get_playback_volume(pcm_element,
+						    SND_MIXER_SCHN_FRONT_RIGHT,
+						    &lr);
+		*l = ll;
+		*r = lr;
+	}
 }
 
 
-
 void alsa_set_volume(int l, int r)
 {
 	if (!pcm_element)
 		return;
 
-	snd_mixer_selem_set_playback_volume(pcm_element,
-					    SND_MIXER_SCHN_FRONT_LEFT, l);
-	snd_mixer_selem_set_playback_volume(pcm_element,
-					    SND_MIXER_SCHN_FRONT_RIGHT, r);
+	if (alsa_cfg.soft_volume)
+	{
+		alsa_cfg.vol.left = l;
+		alsa_cfg.vol.right = r;
+	}
+	else
+	{
+		snd_mixer_selem_set_playback_volume(pcm_element,
+						    SND_MIXER_SCHN_FRONT_LEFT, l);
+		snd_mixer_selem_set_playback_volume(pcm_element,
+						    SND_MIXER_SCHN_FRONT_RIGHT, r);
+	}
 }
 
 
@@ -423,15 +473,117 @@ int alsa_get_output_time(void)
 		db = snd_pcm_frames_to_bytes(alsa_pcm, delay);
 	
 	if (db < alsa_total_written)
-		return ((alsa_total_written - db) / alsa_bps);
+		return ((alsa_total_written - db) * 1000 / alsa_bps);
 	return 0;
 }
 
 int alsa_get_written_time(void)
 {
-	return (alsa_total_written / alsa_bps);
+	return (alsa_total_written * 1000 / alsa_bps);
 }
 
+#define STEREO_ADJUST(type, type2, endian)					\
+do {										\
+	type *ptr = data;							\
+	for (i = 0; i < length; i += 4)						\
+	{									\
+		*ptr = type2##_TO_##endian(type2##_FROM_## endian(*ptr) *	\
+					   alsa_cfg.vol.left / 100);		\
+		ptr++;								\
+		*ptr = type2##_TO_##endian(type2##_FROM_##endian(*ptr) *	\
+					   alsa_cfg.vol.right / 100);		\
+		ptr++;								\
+	}									\
+} while (0)
+
+#define MONO_ADJUST(type, type2, endian)					\
+do {										\
+	type *ptr = data;							\
+	for (i = 0; i < length; i += 4)						\
+	{									\
+		*ptr = type2##_TO_##endian(type2##_FROM_## endian(*ptr) *	\
+					   vol / 100);				\
+		ptr++;								\
+	}									\
+} while (0)
+
+#define VOLUME_ADJUST(type, type2, endian)		\
+do {							\
+	if (channels == 2)				\
+		STEREO_ADJUST(type, type2, endian);	\
+	else						\
+		MONO_ADJUST(type, type2, endian);	\
+} while (0)	
+
+#define STEREO_ADJUST8(type)				\
+do {							\
+	type *ptr = data;				\
+	for (i = 0; i < length; i += 2)			\
+	{						\
+		*ptr = *ptr * alsa_cfg.vol.left / 100;	\
+		ptr++;					\
+		*ptr = *ptr * alsa_cfg.vol.right / 100;	\
+		ptr++;					\
+	}						\
+} while (0)
+
+#define MONO_ADJUST8(type)			\
+do {						\
+	type *ptr = data;			\
+	for (i = 0; i < length; i += 4)		\
+	{					\
+		*ptr = *ptr * vol / 100;	\
+		ptr++;				\
+	}					\
+} while (0)
+
+#define VOLUME_ADJUST8(type)			\
+do {						\
+	if (channels == 2)			\
+		STEREO_ADJUST8(type);		\
+	else					\
+		MONO_ADJUST8(type);		\
+} while (0)	
+
+
+static void volume_adjust(void* data, int length, AFormat fmt, int channels)
+{
+	int i, vol;
+
+	if ((alsa_cfg.vol.left == 100 && alsa_cfg.vol.right == 100) ||
+	    (channels == 1 &&
+	     (alsa_cfg.vol.left == 100 || alsa_cfg.vol.right == 100)))
+		return;
+
+	vol = MAX(alsa_cfg.vol.left, alsa_cfg.vol.right);
+	
+	switch (fmt)
+	{
+		case FMT_S16_LE:
+			VOLUME_ADJUST(gint16, GINT16, LE);
+			break;
+		case FMT_U16_LE:
+			VOLUME_ADJUST(guint16, GUINT16, LE);
+			break;
+		case FMT_S16_BE:
+			VOLUME_ADJUST(gint16, GINT16, BE);
+			break;
+		case FMT_U16_BE:
+			VOLUME_ADJUST(guint16, GUINT16, BE);
+			break;
+		case FMT_S8:
+			VOLUME_ADJUST8(gint8);
+			break;
+		case FMT_U8:
+			VOLUME_ADJUST8(guint8);
+			break;
+		default:
+			g_warning("volue_adjust(): unhandled format: %d", fmt);
+			break;
+	}
+}
+
+
 void alsa_write(gpointer data, int length)
 {
 	EffectPlugin *ep;
@@ -439,6 +591,8 @@ void alsa_write(gpointer data, int length)
 	if (paused)
 		return;
 
+	force_start = FALSE;
+
 	if (effects_enabled() && (ep = get_current_effect_plugin()))
 	{
 		int new_freq = inputf->rate;
@@ -487,6 +641,9 @@ void alsa_write(gpointer data, int length)
 						     effectf->rate,
 						     outputf->rate);
 
+	if (alsa_cfg.soft_volume)
+		volume_adjust(data, length, outputf->xmms_format, outputf->channels);
+
 	if (mmap)
 		alsa_mmap_audio(data, length);
 	else
@@ -524,35 +681,45 @@ static void alsa_write_audio(gpointer data, int length)
 static void alsa_mmap_audio(char *data, int length)
 {
 	int cnt = 0, err;
-	snd_pcm_uframes_t offset, frames;
-	snd_pcm_sframes_t avail;
+	snd_pcm_uframes_t offset, frames, frame;
 	const snd_pcm_channel_area_t *chan_areas = areas;
+	int channel_offset = 0, channel;
+	ssize_t sample_size, offset_bytes, step;
+
+	alsa_get_avail();
 
 	while (length > 0)
 	{
-		avail = alsa_get_avail();
-			
-		if (avail < alsa_period_size)
-		{
-			if ((err = snd_pcm_wait(alsa_pcm, 1)) < 0)
-				g_warning("alsa_mmap_audio(): snd_pcm_wait() "
-					  "failed: %s", snd_strerror(-err));
-			continue;
-		}
-			
 		frames = snd_pcm_bytes_to_frames(alsa_pcm, length);
 		if ((err = snd_pcm_mmap_begin(alsa_pcm, &chan_areas, &offset, &frames) < 0))
 			g_warning("alsa_mmap_audio(): snd_pcm_mmap_begin() "
 				  "failed: %s", snd_strerror(-err));
 
 		cnt = snd_pcm_frames_to_bytes(alsa_pcm, frames);
-			
-		memcpy((char*) chan_areas[0].addr + snd_pcm_frames_to_bytes(alsa_pcm, offset),
-		       data, cnt);
+
+		sample_size = snd_pcm_samples_to_bytes(alsa_pcm, 1);
+		step = chan_areas[0].step / 8;
+		offset_bytes = offset * step;
+
+		for (frame = 0; frame < frames; frame++)
+		{
+			for (channel = 0; channel < outputf->channels; channel++)
+			{
+				char *ptr = chan_areas[channel].addr;
+				memcpy(ptr + chan_areas[channel].first / 8 +
+				       offset_bytes,
+				       data + channel_offset, sample_size);
+				channel_offset += sample_size;
+			}
+			offset_bytes += step;
+		}
 			
 		err = snd_pcm_mmap_commit(alsa_pcm, offset, frames);
-		if (err < 0)
+		if (err == -EPIPE)
 			xrun_recover();
+		else if (err < 0)
+			g_warning("alsa_mmap_audio(): snd_pcm_mmap_commit() "
+				  "failed: %s", snd_strerror(-err));
 		else if (err != frames)
 			g_warning("alsa_mmap_audio(): snd_pcm_mmap_commit "
 				  "returned %d, expected %d", err, (int)frames);
@@ -560,17 +727,6 @@ static void alsa_mmap_audio(char *data, int length)
 		alsa_total_written += cnt;
 		
 		length -= cnt;
-		data += cnt;
-		
-		if (first_write)
-		{
-			first_write = 0;
-			if ((err = snd_pcm_start(alsa_pcm)) < 0)
-				g_warning("alsa_mmap_audio(): snd_pcm_start "
-					  "failed: %s\n", snd_strerror(-err));
-			else
-				debug("Stream started\n");
-		}
 	}
 }
 
@@ -584,24 +740,21 @@ int alsa_open(AFormat fmt, int rate, int nch)
 		snd_output_stdio_attach(&logs, stdout, 0);
 
 	mmap = alsa_cfg.mmap;
-	
+
 	if (alsa_setup(inputf) < 0)
 	{
 		alsa_close();
 		return 0;
 	}
 
-	if (alsa_setup_mixer() < 0)
-	{
-		alsa_close();
-		return 0;
-	}
+	alsa_setup_mixer();
 
 	convertb = xmms_convert_buffers_new();
 	
 	alsa_total_written = 0;
-	going = first_write = 1;
+	going = TRUE;
 	paused = FALSE;
+	force_start = FALSE;
 	
 	snd_pcm_prepare(alsa_pcm);
 	
@@ -622,7 +775,15 @@ static struct snd_format * snd_format_from_xmms(AFormat fmt, int rate, int chann
 			f->format = format_table[i].alsa;
 			break;
 		}
-			
+
+	/* Get rid of _NE */
+	for (i = 0; i < sizeof(format_table) / sizeof(format_table[0]); i++)
+		if (format_table[i].alsa == f->format)
+		{
+			f->xmms_format = format_table[i].xmms;
+			break;
+		}
+
 
 	f->rate = rate;
 	f->channels = channels;
@@ -647,8 +808,7 @@ static int alsa_setup(struct snd_format *f)
 	snd_pcm_sw_params_t *swparams;
 	int alsa_buffer_time, bits_per_sample;
 	unsigned int alsa_period_time;
-	snd_pcm_uframes_t alsa_buffer_size;
-	char *device;
+	snd_pcm_uframes_t alsa_buffer_size, alsa_period_size;
 
 	debug("alsa_setup");
 
@@ -659,23 +819,19 @@ static int alsa_setup(struct snd_format *f)
 	outputf = snd_format_from_xmms(effectf->xmms_format,
 				       effectf->rate,
 				       effectf->channels);
-	g_message("output format: %d", outputf->xmms_format);
-	if (alsa_cfg.use_user_device)
-		device = g_strdup(alsa_cfg.user_device);
-	else
-		device = g_strdup_printf("hw:%d,%d", alsa_cfg.audio_card,
-					 alsa_cfg.audio_device);
 
-	debug("Opening device: %s\n", device);
-	if ((err = snd_pcm_open(&alsa_pcm, device, SND_PCM_STREAM_PLAYBACK, 0)) < 0)
+	debug("Opening device: %s", alsa_cfg.pcm_device);
+	/* FIXME: Can snd_pcm_open() return EAGAIN? */
+	if ((err = snd_pcm_open(&alsa_pcm, alsa_cfg.pcm_device,
+				SND_PCM_STREAM_PLAYBACK,
+				SND_PCM_NONBLOCK)) < 0)
 	{
 		g_warning("alsa_setup(): Failed to open pcm device (%s): %s",
-			  device, snd_strerror(-err));
-		g_free(device);
+			  alsa_cfg.pcm_device, snd_strerror(-err));
 		alsa_pcm = NULL;
 		return -1;
 	}
-	g_free(device);
+	snd_pcm_nonblock(alsa_pcm, FALSE);
 
 	if (alsa_cfg.debug)
 	{
@@ -699,7 +855,7 @@ static int alsa_setup(struct snd_format *f)
 			  "playback: %s", snd_strerror(-err));
 		return -1;
 	}
-	
+
 	if (mmap &&
 	    (err = snd_pcm_hw_params_set_access(alsa_pcm, hwparams,
 						SND_PCM_ACCESS_MMAP_INTERLEAVED)) < 0)
@@ -790,7 +946,7 @@ static int alsa_setup(struct snd_format *f)
 		if (alsa_frequency_convert_func == NULL)
 			return -1;
 	}
-	
+
 	alsa_buffer_time = alsa_cfg.buffer_time * 1000;
 	if ((err = snd_pcm_hw_params_set_buffer_time_near(alsa_pcm, hwparams,
 							  &alsa_buffer_time, 0)) < 0)
@@ -798,8 +954,8 @@ static int alsa_setup(struct snd_format *f)
 		g_warning("alsa_setup(): Set buffer time failed: %s.",
 			  snd_strerror(-err));
 		return -1;
-	}      
-	
+	}
+
 	alsa_period_time = alsa_cfg.period_time * 1000;
 	if ((err = snd_pcm_hw_params_set_period_time_near(alsa_pcm, hwparams,
 							  &alsa_period_time, 0)) < 0)
@@ -819,23 +975,30 @@ static int alsa_setup(struct snd_format *f)
 
 	if ((err = snd_pcm_hw_params_get_buffer_size(hwparams, &alsa_buffer_size)) < 0)
 	{
-	    g_warning ("alsa_setup(): snd_pcm_hw_params_get_buffer_size() failed: %s",
-		       snd_strerror(-err));
-	    return -1;
+		g_warning("alsa_setup(): snd_pcm_hw_params_get_buffer_size() "
+			  "failed: %s",
+			  snd_strerror(-err));
+		return -1;
 	}
-	
+
 	if ((err = snd_pcm_hw_params_get_period_size(hwparams, &alsa_period_size, 0)) < 0)
 	{
-	    g_warning ("alsa_setup(): snd_pcm_hw_params_get_period_size() failed: %s",
-		       snd_strerror(-err));
-	    return -1;
+		g_warning("alsa_setup(): snd_pcm_hw_params_get_period_size() "
+			  "failed: %s",
+			  snd_strerror(-err));
+		return -1;
 	}
-	
+
 	alsa_can_pause = snd_pcm_hw_params_can_pause(hwparams);
 
 	snd_pcm_sw_params_alloca(&swparams);
 	snd_pcm_sw_params_current(alsa_pcm, swparams);
-	
+
+	/* This has effect for non-mmap only */
+	if ((err = snd_pcm_sw_params_set_start_threshold(alsa_pcm,
+			swparams, alsa_buffer_size - alsa_period_size) < 0))
+		g_warning("alsa_setup(): setting start "
+			  "threshold failed: %s", snd_strerror(-err));
 	if (snd_pcm_sw_params(alsa_pcm, swparams) < 0)
 	{
 		g_warning("alsa_setup(): Unable to install sw params");
@@ -847,15 +1010,14 @@ static int alsa_setup(struct snd_format *f)
 		snd_pcm_sw_params_dump(swparams, logs);
 		snd_pcm_dump(alsa_pcm, logs);
 	}
-	
+
 	bits_per_sample = snd_pcm_format_physical_width(outputf->format);
-	alsa_bps = (outputf->rate * bits_per_sample * outputf->channels) / 8000;
+	alsa_bps = (outputf->rate * bits_per_sample * outputf->channels) >> 3;
 
 	if (mmap)
 	{
 		int chn;
 		buffer = g_malloc(alsa_period_size * bits_per_sample / 8 * outputf->channels);
-		
 		areas = g_malloc0(outputf->channels * sizeof(snd_pcm_channel_area_t));
 		
 		for (chn = 0; chn < outputf->channels; chn++)
@@ -866,7 +1028,7 @@ static int alsa_setup(struct snd_format *f)
 		}
 	}
 
-	debug("Device setup: buffer time: %i, size: %i frames.", alsa_buffer_time,
+	debug("Device setup: buffer time: %i, size: %i.", alsa_buffer_time,
 	      snd_pcm_frames_to_bytes(alsa_pcm, alsa_buffer_size));
 	debug("bits per sample: %i; frame size: %i; Bps: %i",
 	      bits_per_sample, snd_pcm_frames_to_bytes(alsa_pcm, 1), alsa_bps);
