@@ -126,7 +126,7 @@ void esdout_configure(void)
 				     esd_cfg.use_oss_mixer);
 	gtk_box_pack_start(GTK_BOX(server_btn_hbox),
 			   server_oss_mixer, TRUE, TRUE, 0);
-#if !defined(HAVE_SYS_SOUNDCARD_H) && !defined(HAVE_MACHINE_SOUNDCARD_H)
+#if HAVE_OSS
 	gtk_widget_set_sensitive(server_oss_mixer, FALSE);
 #endif
 	server_hbox = gtk_hbox_new(FALSE, 5);
