@@ -778,6 +778,13 @@ gint abuffer_open(AFormat fmt, gint rate, gint nch)
 		stream_attrs.output_precision=16;
 		stream_attrs.is_signed = TRUE;
 	}
+	else if (!strcmp(aud_dev.name, "SUNW,audioens"))
+	{
+		/* Ensoniq1371/1373 and Creative Labs 5880 audio controller */
+		stream_attrs.output_precision=stream_attrs.precision;
+		stream_attrs.is_signed = TRUE;
+		sun_cfg.channel_flags = 0;
+	}
 	else if (!strcmp(aud_dev.name, "SUNW,sb16") || !strcmp(aud_dev.name, "TOOLS,sbpci"))
 	{
 		/* Sound blaster (or compatible, inc AWE32/64) */
