@@ -1,6 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999-2002  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,9 +18,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#if defined(HAVE_CONFIG_H)
 #include "config.h"
-#endif
 
 #include "../xmms/i18n.h"
 
@@ -110,38 +109,40 @@ static char *ofolder[] =
 	"                ",
 	"                "};
 
+#define NODE_SPACING 4
+
 static GdkPixmap *folder_pixmap = NULL, *ofolder_pixmap;
 static GdkBitmap *folder_mask, *ofolder_mask;
 
-typedef struct
+struct dirnode
 {
-	gboolean scanned;
-	gchar *path;
-}
-DirNode;
+	unsigned int scanned : 1;
+	char *path;
+};
 
-static gboolean check_for_subdir(gchar * path)
+static gboolean check_for_subdir(char *path)
 {
 	DIR *dir;
 	struct dirent *dirent;
 	struct stat statbuf;
-	gchar *npath;
+	char *npath;
 
 	if ((dir = opendir(path)) != NULL)
 	{
 		while ((dirent = readdir(dir)) != NULL)
 		{
-			if (dirent->d_name[0] != '.')
+			if (dirent->d_name[0] == '.')
+				continue;
+
+			npath = g_strconcat(path, dirent->d_name, NULL);
+			if (stat(npath, &statbuf) != -1 &&
+			    S_ISDIR(statbuf.st_mode))
 			{
-				npath = g_strconcat(path, dirent->d_name, NULL);
-				if (stat(npath, &statbuf) != -1 && S_ISDIR(statbuf.st_mode))
-				{
-					g_free(npath);
-					closedir(dir);
-					return TRUE;
-				}
 				g_free(npath);
+				closedir(dir);
+				return TRUE;
 			}
+			g_free(npath);
 		}
 		closedir(dir);
 	}
@@ -150,84 +151,106 @@ static gboolean check_for_subdir(gchar * path)
 
 static void destroy_cb(gpointer data)
 {
-	DirNode *node = data;
+	struct dirnode *node = data;
 
 	g_free(node->path);
 	g_free(node);
 }
 
-static void expand_cb(GtkWidget * widget, GtkCTreeNode * parent_node)
+static void add_dir(GtkCTree *tree, GtkCTreeNode *pnode, char* parent, char *dir)
 {
-	DIR *dir;
-	struct dirent *dirent;
-	gchar *path, *text, *dummy = "dummy";
 	struct stat statbuf;
-	GtkCTreeNode *node, *sub_node;
-	DirNode *parent_dirnode, *dirnode;
-	gboolean has_subdir = FALSE;
+	char *path;
 
-	parent_dirnode = gtk_ctree_node_get_row_data(GTK_CTREE(widget), parent_node);
+	/* Don't show hidden dirs, nor . and .. */
+	if (dir[0] == '.')
+		return;
+
+	path = g_strconcat(parent, dir, NULL);
+	if (stat(path, &statbuf) != -1 && S_ISDIR(statbuf.st_mode))
+	{
+		gboolean has_subdir;
+		char *text = "";
+		GtkCTreeNode *node;
+		struct dirnode *dirnode = g_malloc0(sizeof (struct dirnode));
+		dirnode->path = g_strconcat(path, "/", NULL);
+ 		has_subdir = check_for_subdir(dirnode->path);
+		node = gtk_ctree_insert_node(tree, pnode, NULL, &dir,
+					     NODE_SPACING, folder_pixmap,
+					     folder_mask, ofolder_pixmap,
+					     ofolder_mask, !has_subdir, FALSE);
+		gtk_ctree_node_set_row_data_full(tree, node, dirnode,
+						 destroy_cb);
+		if (has_subdir)
+			gtk_ctree_insert_node(tree, node, NULL, &text,
+					      NODE_SPACING, NULL, NULL,
+					      NULL, NULL, FALSE, FALSE);
+	}
+	g_free(path);
+}
+
+static void expand_cb(GtkWidget *widget, GtkCTreeNode *parent_node)
+{
+	struct dirent *dirent;
+	GtkCTree *tree = GTK_CTREE(widget);
+	struct dirnode *parent_dirnode;
+
+	parent_dirnode = gtk_ctree_node_get_row_data(tree, parent_node);
 	if (!parent_dirnode->scanned)
 	{
+		DIR *dir;
+
 		gtk_clist_freeze(GTK_CLIST(widget));
-		node = gtk_ctree_find_by_row_data(GTK_CTREE(widget), parent_node, NULL);
-		gtk_ctree_remove_node(GTK_CTREE(widget), node);
+		gtk_ctree_remove_node(tree,
+				      GTK_CTREE_ROW(parent_node)->children);
 		if ((dir = opendir(parent_dirnode->path)) != NULL)
 		{
 			while ((dirent = readdir(dir)) != NULL)
 			{
-				path = g_strconcat(parent_dirnode->path, dirent->d_name, NULL);
-				if (stat(path, &statbuf) != -1 && S_ISDIR(statbuf.st_mode) && dirent->d_name[0] != '.')
-				{
-					dirnode = g_malloc0(sizeof (DirNode));
-					dirnode->path = g_strconcat(path, "/", NULL);
-					text = dirent->d_name;
-					if (check_for_subdir(dirnode->path))
-						has_subdir = TRUE;
-					else
-						has_subdir = FALSE;
-					node = gtk_ctree_insert_node(GTK_CTREE(widget), parent_node, NULL, &text, 4, folder_pixmap, folder_mask, ofolder_pixmap, ofolder_mask, !has_subdir, FALSE);
-					gtk_ctree_node_set_row_data_full(GTK_CTREE(widget), node, dirnode, destroy_cb);
-					if (has_subdir)
-						sub_node = gtk_ctree_insert_node(GTK_CTREE(widget), node, NULL, &dummy, 4, NULL, NULL, NULL, NULL, FALSE, FALSE);
-				}
-				g_free(path);
+				add_dir(tree, parent_node,
+					parent_dirnode->path, dirent->d_name);
 			}
 			closedir(dir);
-			gtk_ctree_sort_node(GTK_CTREE(widget), parent_node);
+			gtk_ctree_sort_node(tree, parent_node);
 		}
 		gtk_clist_thaw(GTK_CLIST(widget));
 		parent_dirnode->scanned = TRUE;
 	}
 }
 
-static void select_row_cb(GtkWidget * widget, gint row, gint column, GdkEventButton * bevent, gpointer data)
+static void select_row_cb(GtkWidget *widget, int row, int column, GdkEventButton *bevent, gpointer data)
 {
-	DirNode *dirnode;
+	struct dirnode *dirnode;
 	GtkCTreeNode *node;
-	void (*handler) (gchar *);
+	void (*handler) (char *);
 
-	if (bevent)
+	if (bevent && bevent->type == GDK_2BUTTON_PRESS)
 	{
-		if (bevent->type == GDK_2BUTTON_PRESS)
-		{
-			node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
-			dirnode = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
-			handler = (void (*)(gchar *)) gtk_object_get_user_data(GTK_OBJECT(widget));
-			if (handler)
-				handler(dirnode->path);
-		}
+		node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
+		dirnode = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
+		handler = gtk_object_get_user_data(GTK_OBJECT(widget));
+		if (handler)
+			handler(dirnode->path);
 	}
-
 }
 
-static void ok_clicked(GtkWidget * widget, GtkWidget * tree)
+static void show_cb(GtkWidget *widget, gpointer data)
+{
+	GtkCTree *tree = GTK_CTREE(data);
+	GtkCTreeNode *node = gtk_object_get_data(GTK_OBJECT(tree),
+						 "selected_node");
+
+	if (node)
+		gtk_ctree_node_moveto(tree, node, -1, 0.6, 0);
+}
+
+static void ok_clicked(GtkWidget *widget, GtkWidget *tree)
 {
 	GtkCTreeNode *node;
-	DirNode *dirnode;
+	struct dirnode *dirnode;
 	GList *list_node;
 	GtkWidget *window;
-	void (*handler) (gchar *);
+	void (*handler) (char *);
 
 	window = gtk_object_get_user_data(GTK_OBJECT(widget));
 	gtk_widget_hide(window);
@@ -236,7 +259,7 @@ static void ok_clicked(GtkWidget * widget, GtkWidget * tree)
 	{
 		node = list_node->data;
 		dirnode = gtk_ctree_node_get_row_data(GTK_CTREE(tree), node);
-		handler = (void (*)(gchar *)) gtk_object_get_user_data(GTK_OBJECT(tree));
+		handler = gtk_object_get_user_data(GTK_OBJECT(tree));
 		if (handler)
 			handler(dirnode->path);
 		list_node = g_list_next(list_node);
@@ -245,24 +268,16 @@ static void ok_clicked(GtkWidget * widget, GtkWidget * tree)
 
 }
 
-static int filetreeent_compare_func(const void *a, const void *b)
+GtkWidget *xmms_create_dir_browser(char *title, char *current_path, GtkSelectionMode mode, void (*handler) (char *))
 {
-	if (!a || !b || !((DirNode *) a)->path)
-		return -1;
-	return strcmp(((DirNode *) a)->path, (gchar *) b);
-}
-
-GtkWidget *xmms_create_dir_browser(gchar * title, gchar * current_path, GtkSelectionMode mode, void (*handler) (gchar *))
-{
-	GtkWidget *window, *scroll_win, *tree, *vbox, *bbox, *ok, *cancel,
-	         *sep;
-	gchar *root_text = "/", *node_text = "dummy";
-	gchar *currentdir, *pos, *tpath, *tpathnew;
-	GtkCTreeNode *root_node, *node, *nextnode;
-	DirNode *dirnode;
-	gboolean leaf;
+	GtkWidget *window, *scroll_win, *tree, *vbox, *bbox, *ok, *cancel, *sep;
+	char *root_text = "/", *text = "";
+	GtkCTreeNode *root_node, *node, *selected_node = NULL;
+	GtkCTree *ctree;
+	struct dirnode *dirnode;
 
 	window = gtk_window_new(GTK_WINDOW_DIALOG);
+	gtk_window_set_default_size(GTK_WINDOW(window), 300, 400);
 	gtk_window_set_title(GTK_WINDOW(window), title);
 	gtk_container_border_width(GTK_CONTAINER(window), 10);
 
@@ -270,33 +285,50 @@ GtkWidget *xmms_create_dir_browser(gchar * title, gchar * current_path, GtkSelec
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
 	scroll_win = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_widget_set_usize(scroll_win, 250, 200);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll_win),
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start(GTK_BOX(vbox), scroll_win, TRUE, TRUE, 0);
 	gtk_widget_show(scroll_win);
 
 	gtk_widget_realize(window);
 	if (!folder_pixmap)
 	{
-		folder_pixmap = gdk_pixmap_create_from_xpm_d(window->window, &folder_mask, NULL, folder);
-		ofolder_pixmap = gdk_pixmap_create_from_xpm_d(window->window, &ofolder_mask, NULL, ofolder);
+		folder_pixmap = gdk_pixmap_create_from_xpm_d(window->window,
+							     &folder_mask,
+							     NULL, folder);
+		ofolder_pixmap = gdk_pixmap_create_from_xpm_d(window->window,
+							      &ofolder_mask,
+							      NULL, ofolder);
 	}
 
 	tree = gtk_ctree_new(1, 0);
+	ctree = GTK_CTREE(tree);
 	gtk_clist_set_column_auto_resize(GTK_CLIST(tree), 0, TRUE);
 	gtk_clist_set_selection_mode(GTK_CLIST(tree), mode);
-	gtk_ctree_set_line_style(GTK_CTREE(tree), GTK_CTREE_LINES_DOTTED);
-	gtk_signal_connect(GTK_OBJECT(tree), "tree_expand", GTK_SIGNAL_FUNC(expand_cb), NULL);
-	gtk_signal_connect(GTK_OBJECT(tree), "select_row", GTK_SIGNAL_FUNC(select_row_cb), NULL);
+	gtk_ctree_set_line_style(ctree, GTK_CTREE_LINES_DOTTED);
+	gtk_signal_connect(GTK_OBJECT(tree), "tree_expand",
+			   GTK_SIGNAL_FUNC(expand_cb), NULL);
+	gtk_signal_connect(GTK_OBJECT(tree), "select_row",
+			   GTK_SIGNAL_FUNC(select_row_cb), NULL);
+	gtk_signal_connect(GTK_OBJECT(window), "show",
+			   GTK_SIGNAL_FUNC(show_cb), tree);
 	gtk_container_add(GTK_CONTAINER(scroll_win), tree);
-	gtk_object_set_user_data(GTK_OBJECT(tree), (gpointer) handler);
+	gtk_object_set_user_data(GTK_OBJECT(tree), handler);
 
-	root_node = gtk_ctree_insert_node(GTK_CTREE(tree), NULL, NULL, &root_text, 4, folder_pixmap, folder_mask, ofolder_pixmap, ofolder_mask, FALSE, FALSE);
-	dirnode = g_malloc0(sizeof (DirNode));
+	root_node = gtk_ctree_insert_node(ctree, NULL, NULL,
+					  &root_text, NODE_SPACING,
+					  folder_pixmap, folder_mask,
+					  ofolder_pixmap, ofolder_mask,
+					  FALSE, FALSE);
+	dirnode = g_malloc0(sizeof (struct dirnode));
 	dirnode->path = g_strdup("/");
-	gtk_ctree_node_set_row_data_full(GTK_CTREE(tree), root_node, dirnode, destroy_cb);
-	node = gtk_ctree_insert_node(GTK_CTREE(tree), root_node, NULL, &node_text, 4, NULL, NULL, NULL, NULL, TRUE, TRUE);
-	gtk_ctree_expand(GTK_CTREE(tree), root_node);
+	gtk_ctree_node_set_row_data_full(ctree, root_node,
+					 dirnode, destroy_cb);
+	node = gtk_ctree_insert_node(ctree, root_node, NULL,
+				     &text, 4, NULL, NULL, NULL,
+				     NULL, TRUE, TRUE);
+	gtk_ctree_expand(ctree, root_node);
 	gtk_widget_show(tree);
 
 	sep = gtk_hseparator_new();
@@ -312,13 +344,16 @@ GtkWidget *xmms_create_dir_browser(gchar * title, gchar * current_path, GtkSelec
 	GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT);
 	gtk_window_set_default(GTK_WINDOW(window), ok);
 	gtk_box_pack_start(GTK_BOX(bbox), ok, TRUE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(ok), "clicked", GTK_SIGNAL_FUNC(ok_clicked), tree);
+	gtk_signal_connect(GTK_OBJECT(ok), "clicked",
+			   GTK_SIGNAL_FUNC(ok_clicked), tree);
 	gtk_widget_show(ok);
 
 	cancel = gtk_button_new_with_label(_("Cancel"));
 	GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);
 	gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 0);
-	gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(window));
+	gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked",
+				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
+				  GTK_OBJECT(window));
 	gtk_widget_show(cancel);
 
 	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
@@ -327,35 +362,44 @@ GtkWidget *xmms_create_dir_browser(gchar * title, gchar * current_path, GtkSelec
 
 	if (current_path && *current_path)
 	{
-		currentdir = g_strdup(current_path);
-		tpath = g_strdup("/");
-		pos = strtok(currentdir, "/");
-		node = gtk_ctree_find_by_row_data_custom(GTK_CTREE(tree), NULL, "/", filetreeent_compare_func);
-		do
+		char **dir;
+		int i;
+
+		dir = g_strsplit(current_path, "/", 0);
+		node = root_node;
+		for (i = 0; dir[i] != NULL; i++)
 		{
-			tpathnew = g_strconcat(tpath, pos, "/", NULL);
-			g_free(tpath);
-			tpath = tpathnew;
-			nextnode = gtk_ctree_find_by_row_data_custom(GTK_CTREE(tree), node, tpath, filetreeent_compare_func);
-			if (!nextnode)
+			if (dir[i][0] == '\0')
+				continue;
+
+			for (node = GTK_CTREE_ROW(node)->children; node != NULL;
+			     node = GTK_CTREE_ROW(node)->sibling)
+			{
+				char *tmp;
+				if (gtk_ctree_node_get_pixtext(ctree, node, 0,
+							       &tmp, NULL,
+							       NULL, NULL))
+					if (!strcmp(dir[i], tmp))
+						break;
+			}
+			if (!node)
 				break;
-			node = nextnode;
-			pos = strtok(NULL, "/");
-			gtk_ctree_get_node_info(GTK_CTREE(tree), node, NULL, NULL, NULL, NULL, NULL, NULL, &leaf, NULL);
-			if (!leaf && pos)
-				gtk_ctree_expand(GTK_CTREE(tree), node);
+			if (!GTK_CTREE_ROW(node)->is_leaf && dir[i + 1])
+				gtk_ctree_expand(ctree, node);
 			else
 			{
-				gtk_ctree_select(GTK_CTREE(tree), node);
+				selected_node = node;
 				break;
 			}
 		}
-		while (pos);
-		g_free(tpath);
-		g_free(currentdir);
+		g_strfreev(dir);
 	}
-	else
-		gtk_ctree_select(GTK_CTREE(tree), root_node);
+
+	if (!selected_node)
+		selected_node = root_node;
+
+	gtk_ctree_select(ctree, selected_node);
+	gtk_object_set_data(GTK_OBJECT(tree), "selected_node", selected_node);
 
 	return window;
 }
