@@ -110,6 +110,7 @@ void playlist_delete_node(GList *node, gboolean *set_info_text, gboolean *restar
 	/* Caller should hold playlist mutex */
 	PlaylistEntry *entry;
 	GList *playing_song = NULL;
+	GList *queued_song = NULL;
 	/*
 	 * We call g_list_find manually here because
 	 * we don't want an item in the shuffle_list
@@ -142,6 +143,13 @@ void playlist_delete_node(GList *node, gboolean *set_info_text, gboolean *restar
 		 g_list_position(playlist, playing_song) >
 		 g_list_position(playlist, node))
 		*set_info_text = TRUE;
+
+	if ((queued_song = g_list_find(queued_list, entry)) != NULL)
+	{
+		queued_list = g_list_remove_link(queued_list, queued_song);
+		g_list_free_1(queued_song);
+	}
+
 	if (entry->filename)
 		g_free(entry->filename);
 	if (entry->title)
