@@ -152,7 +152,7 @@ static const int playlistwin_sub_menu_entries_num =
 
 GtkItemFactoryEntry playlistwin_popup_menu_entries[] =
 {
-	{N_("/View File Info"), NULL, playlistwin_popup_menu_callback, MISC_FILEINFO, "<Item>"},
+	{N_("/View File Info"), "<control>3", playlistwin_popup_menu_callback, MISC_FILEINFO, "<Item>"},
 	{N_("/Queue - Unqueue"), "Q", playlistwin_popup_menu_callback, MISC_QUEUE, "<Item>"},
 	{N_("/-"), NULL, NULL, 0, "<Separator>"},
 	{N_("/Add"), NULL, NULL, 0, "<Branch>"},
