@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-2001  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2001  Haavard Kvaalen
+ *  Copyright (C) 1999-2003  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -302,10 +302,10 @@ void prefswin_rt_callback(GtkToggleButton * w, gpointer data)
 		_("Realtime priority is a way for XMMS to get a higher\n"
 		  "priority  for CPU time.  This might give less \"skips\".\n\n"
 		  "This requires that XMMS is run with root privileges and\n"
-		  "may, although it's very unusal, lock up your computer.\n"
-		  "Running XMMS with root privilegies might also have other\n"
+		  "may, although it's very unusual, lock up your computer.\n"
+		  "Running XMMS with root privileges might also have other\n"
 		  "security implications.\n\n"
-		  "Using this feature is not recomended.\n"
+		  "Using this feature is not recommended.\n"
 		  "To activate this you need to restart XMMS."),
 		_("Ok"), FALSE, NULL, NULL);
 }
@@ -989,11 +989,11 @@ void create_prefs_window(void)
 	prefswin_option_new_with_label_to_table(&cfg.dim_titlebar, _("Dim titlebar when inactive"), GTK_TABLE(options_table), 0, 3);
 	prefswin_option_new_with_label_to_table(&cfg.show_numbers_in_pl, _("Show numbers in playlist"), GTK_TABLE(options_table), 1, 3);
 
-	prefswin_option_new_with_label_to_table(&cfg.sort_jump_to_file, _("Sort \"Jump to file\" alphabetical"), GTK_TABLE(options_table), 0, 4);
+	prefswin_option_new_with_label_to_table(&cfg.sort_jump_to_file, _("Sort \"Jump to file\" alphabetically"), GTK_TABLE(options_table), 0, 4);
 	prefswin_option_new_with_label_to_table(&cfg.eq_doublesize_linked, _("Equalizer doublesize linked"), GTK_TABLE(options_table), 1, 4);
 
 	options_rt = prefswin_option_new_with_label_to_table(&cfg.use_realtime, _("Use realtime priority when available"), GTK_TABLE(options_table), 0, 5);
-	gtk_tooltips_set_tip(prefswin_tooltips, options_rt, _("Run XMMS with higher priority (not recomended)"), NULL);
+	gtk_tooltips_set_tip(prefswin_tooltips, options_rt, _("Run XMMS with higher priority (not recommended)"), NULL);
 	gtk_signal_connect(GTK_OBJECT(options_rt), "toggled", GTK_SIGNAL_FUNC(prefswin_rt_callback), NULL);
 	prefswin_option_new_with_label_to_table(&cfg.smooth_title_scroll, _("Smooth title scroll"), GTK_TABLE(options_table), 1, 5);
 
@@ -1020,12 +1020,17 @@ void create_prefs_window(void)
 	gtk_container_add(GTK_CONTAINER(options_sw), options_sw_box);
 
 
-	prefswin_option_new_with_label_to_table(&cfg.show_wm_decorations, _("Show windowmanager decorations"), GTK_TABLE(options_table), 0, 7);
-	prefswin_option_new_with_label_to_table(&cfg.use_backslash_as_dir_delimiter, _("Use \'\\\' as a directory delimiter"), GTK_TABLE(options_table), 1, 7);
+	prefswin_option_new_with_label_to_table(&cfg.show_wm_decorations,
+						_("Show window manager decorations"),
+						GTK_TABLE(options_table), 0, 7);
+	opt = prefswin_option_new_with_label_to_table(
+		&cfg.use_backslash_as_dir_delimiter,
+		_("Use \'\\\' as a directory delimiter"),
+		GTK_TABLE(options_table), 1, 7);
+	gtk_tooltips_set_tip(prefswin_tooltips, opt,
+			     _("Recommended if you want to load playlists "
+			       "that was created in MS Windows"), NULL);
 
-#if 0
-	prefswin_option_new_with_label_to_table(&cfg.playlist_transparent, _("Transparent playlist window"), GTK_TABLE(options_table), 0, 8);
-#endif
 	options_mouse_box = gtk_hbox_new(FALSE, 5);
 	options_mouse_label = gtk_label_new(_("Mouse Wheel adjusts Volume by (%)"));
 	gtk_box_pack_start(GTK_BOX(options_mouse_box), options_mouse_label, FALSE, FALSE, 0);
@@ -1035,7 +1040,9 @@ void create_prefs_window(void)
 	gtk_box_pack_start(GTK_BOX(options_mouse_box), prefswin_options_mouse_spin, FALSE, FALSE, 0);
 	gtk_table_attach_defaults(GTK_TABLE(options_table), options_mouse_box, 0, 1, 8, 9);
 	
-	prefswin_option_new_with_label_to_table(&cfg.use_pl_metadata, _("Use meta data in playlists"), GTK_TABLE(options_table), 1, 8);
+	prefswin_option_new_with_label_to_table(&cfg.use_pl_metadata,
+						_("Use meta-data in playlists"),
+						GTK_TABLE(options_table), 1, 8);
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(prefswin_notebook), prefswin_options_vbox, gtk_label_new(_("Options")));
 
@@ -1216,7 +1223,6 @@ void add_output_plugins(GtkOptionMenu *omenu)
 		gtk_widget_set_sensitive(prefswin_audio_oabout, TRUE);
 	else
 		gtk_widget_set_sensitive(prefswin_audio_oabout, FALSE);
-
 }
 
 void add_effect_plugins(GtkCList *clist)
