@@ -1,6 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999-2004  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -57,13 +58,13 @@ static void ice_handler(gpointer data, gint source, GdkInputCondition condition)
 	IceProcessMessages(data, NULL, NULL);
 }
 
-void sm_init(int argc, char **argv, const char *previous_session_id)
+const char * sm_init(int argc, char **argv, const char *previous_session_id)
 {
 	SmcCallbacks smcall;
 	char errstr[256];
 
 	if (!getenv("SESSION_MANAGER"))
-		return;
+		return NULL;
 
 	memset(&smcall, 0, sizeof(smcall));
 	smcall.save_yourself.callback = sm_save_yourself;
@@ -134,6 +135,8 @@ void sm_init(int argc, char **argv, const char *previous_session_id)
 
 		g_free(userid_val.value);
 	}
+
+	return session_id;
 }
 
 void sm_cleanup(void)
