@@ -668,9 +668,9 @@ static void filebrowser_entry_changed(GtkEditable *entry, gpointer data)
 		filebrowser_changed(GTK_FILE_SELECTION(data));
 }
 
- static void filebrowser_dir_select(GtkCList *clist, int row, int col, GdkEventButton *event, gpointer data)
+static void filebrowser_dir_select(GtkCList *clist, int row, int col, GdkEventButton *event, gpointer data)
 {
-	if (event->type == GDK_2BUTTON_PRESS)
+	if (event && event->type == GDK_2BUTTON_PRESS)
 		gtk_idle_add(filebrowser_idle_changed, data);
 }
 
@@ -681,9 +681,19 @@ gboolean util_filebrowser_is_dir(GtkFileSelection * filesel)
 	gboolean retv = FALSE;
 	
 	text = g_strdup(gtk_file_selection_get_filename(filesel));
+	if (strlen(text) == 0)
+	{
+		/*
+		 * This is a weird special case.  If the "current" dir
+		 * don't exist, gtk_file_selection_get_filename()
+		 * return an empty string, even if the user typed in a
+		 * dir.
+		 */
+		g_free(text);
+		text = strdup(gtk_entry_get_text(GTK_ENTRY(filesel->selection_entry)));
+	}
 
-	if (stat(text, &buf) == 0 &&
-	    S_ISDIR(buf.st_mode))
+	if ((stat(text, &buf) == 0 && S_ISDIR(buf.st_mode)) || strlen(text) == 0)
 	{
 		/* Selected directory */
 		int len = strlen(text);
@@ -892,7 +902,7 @@ GtkWidget * util_create_filebrowser(gboolean play_button)
 		 */
 		arg.name = "GtkWidget::parent";
 		gtk_object_arg_get(GTK_OBJECT(fb->ok_button), &arg, NULL);
-		button = gtk_button_new_with_label("Add");
+		button = gtk_button_new_with_label(_("Add"));
 		gtk_signal_connect(GTK_OBJECT(button), "clicked",
 				   filebrowser_add, filebrowser);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
